package com.xydl.cac.iec104;

import com.xydl.cac.entity.Iec104Point;
import com.xydl.cac.model.StaticVariable;
import com.xydl.cac.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.openmuc.j60870.*;
import org.openmuc.j60870.ie.*;

import java.io.EOFException;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

@Slf4j
public class ServerConnectionEventListener implements ConnectionEventListener {
    Connection connection;
    int connectionId;
    boolean selected = false;

    public ServerConnectionEventListener(Connection connection, int connectionId) {
        this.connection = connection;
        this.connectionId = connectionId;
    }

    @Override
    public void newASdu(Connection connection, ASdu aSdu) {
        log.info("Got new ASdu, type=" + aSdu.getTypeIdentification());
        InformationObject informationObject = null;
        try {
            switch (aSdu.getTypeIdentification()) {
                // interrogation command
                case C_IC_NA_1:
                    log.debug("Got interrogation command (100). Will send scaled measured values.");
                    connection.sendConfirmation(aSdu);

                    this.sendAll104(connection);

                    connection.sendActivationTermination(aSdu);
                    break;
                case C_SC_NA_1:
                    informationObject = aSdu.getInformationObjects()[0];
                    IeSingleCommand singleCommand = (IeSingleCommand) informationObject
                            .getInformationElements()[0][0];

                    if (informationObject.getInformationObjectAddress() != 5000) {
                        break;
                    }
                    if (singleCommand.isSelect()) {
                        log.debug("Got single command (45) with select true. Select command.");
                        selected = true;
                        connection.sendConfirmation(aSdu);
                    } else if (!singleCommand.isSelect() && selected) {
                        log.debug("Got single command (45) with select false. Execute selected command.");
                        selected = false;
                        connection.sendConfirmation(aSdu);
                    } else {
                        log.debug("Got single command (45) with select false. But no command is selected, no execution.");
                    }
                    break;
                case C_CS_NA_1:
                    IeTime56 ieTime56 = new IeTime56(System.currentTimeMillis());
                    log.debug("Got Clock synchronization command (103). Send current time: " + ieTime56.toString());
                    connection.synchronizeClocks(aSdu.getCommonAddress(), ieTime56);
                    break;
                case C_SE_NB_1:
                    log.debug("Got Set point command, scaled value (49)");
                    break;
                default:
                    log.debug("Got unknown request: " + aSdu.toString(),
                            ". Send negative confirm with CoT UNKNOWN_TYPE_ID(44)");
                    connection.sendConfirmation(aSdu, aSdu.getCommonAddress(), true,
                            CauseOfTransmission.UNKNOWN_TYPE_ID);
            }

        } catch (EOFException e) {
            log.debug("Will quit listening for commands on connection (" + connectionId,
                    ") because socket was closed.");
        } catch (Exception e) {
            log.debug("Will quit listening for commands on connection (" + connectionId, ") because of error: ",
                    e.getMessage());
        }
    }

    private void sendAll104(Connection conn) throws Exception {
        List<Iec104Point> list = StaticVariable.point104_Cache;
        for (Iec104Point item : list) {
            this.sendOne104(conn, item);
        }
    }

    private void sendOne104(Connection conn, Iec104Point item) throws Exception {
        HashMap<String, String> map = StaticVariable.sensorLastDataMap.get(item.getEqmid());
        String value = map.get(item.getField());
        String time = map.get("acquisitionTime");
        if (value != null && time != null) {
            Date date = DateUtil.parse(time);
            Float f = Float.valueOf(value);
            InformationObject infoObj = new InformationObject(item.getSadr(), new InformationElement[][]{
                    {new IeShortFloat(f),
                            new IeQuality(false, false, false, false, false),
                            new IeTime56(date.getTime())}
            });
            ASdu obj = new ASdu(ASduType.M_ME_TF_1, true, CauseOfTransmission.INTERROGATED_BY_STATION,
                    false, false, 0, 1, infoObj);
            conn.send(obj);
            log.info("104服务端发送: sadr=" + item.getSadr() + ", value=" + value + ", time=" + time);
        } else {
            log.info("数据不全,104服务端不发送: sadr=" + item.getSadr() + ", value=" + value + ", time=" + time);
        }
    }

    @Override
    public void connectionClosed(Connection connection, IOException e) {

    }

    @Override
    public void dataTransferStateChanged(Connection connection, boolean b) {

    }
}
